#include <iostream>
#include <string>
#include <fstream>
#include <map>
//#include <tr1/unordered_map>
#include "bep.hpp"

#include "mytimer.h"

using namespace std;

int main(int argc, char *argv[])
{
	if (argc != 3) {
		fprintf(stderr, "%s keylist index\n", argv[0]);
		return -1;
	}

	bep_tool::bep<unsigned int> bp;
	vector<string> keys;		// for check
	{
		ifstream ifs(argv[1]);
		if (!ifs) {
			fprintf(stderr, "cannot open %s\n", argv[1]);
			return -1;
		}
		string line;
		unsigned int num = 0;
		while (getline(ifs, line)) {
			keys.push_back(line);
			bp[line] = num++;
		}
	}

	// check
	for (size_t i = 0; i < keys.size(); i++) {
		if (bp[keys[i]] != i) {
			fprintf(stderr, "bep check error i:%u key:%s retVal:%u\n", (unsigned int) i, keys[i].c_str(), bp[keys[i]]);
			break;
		}
	}

	// save load check
	{
		// you need to save vals by yourself
		std::vector<unsigned int> vals;
		if (!bp.save(argv[2], vals)) {
			fprintf(stderr, "save error\n");
			return -1;
		}
		bep_tool::bep<unsigned int> bp2;
		if (!bp2.load(argv[2], vals)) {
			fprintf(stderr, "load error\n");
		}
		for (size_t i = 0; i < keys.size(); i++) {
			const string& key = keys[i];
			if (bp[key] != bp2[key]) {
				fprintf(stderr, "save read error i=%d, key:%s bp:%u bp2:%u\n", i, key.c_str(), bp[key], bp2[key]);
				return -1;
			}
		}
	}


	// time check
	size_t sum = 0;
	double start = gettimeofday_sec();
	for (size_t i = 0; i < keys.size(); i++) {
		sum += bp[keys[i]];
	}
	double time = gettimeofday_sec() - start;
	printf("%.4f %.4f usec\n", time, time / keys.size() * 1000 * 1000);

	if (sum == 0) {
		printf(" \n");			// disallow optimization
	}
	return 0;
}

