#include <iostream>
#include <string>
#include <fstream>
#include "bhash.hpp"

#include "mytimer.h"

int main(int argc, char *argv[])
{
	if (argc != 2) {
		fprintf(stderr, "%s keylist\n", argv[0]);
		return -1;
	}
	const char *name = argv[1];
	typedef std::vector<std::string> StringSet;
	StringSet all;
	std::ifstream ifs(name, std::ios::binary);
	if (!ifs) {
		fprintf(stderr, "cannot open %s\n", name);
		return -1;
	}
	std::string line;
	while (std::getline(ifs, line)) {
		all.push_back(line);
	}

	bep_tool::bhash bh;
	if (!bh.build(all)) {
		return -1;
	}
	std::vector<bool> hashValue(all.size());
	std::fill(hashValue.begin(), hashValue.end(), false);
	puts("check");
	for (StringSet::const_iterator i = all.begin(), e = all.end(); i != e; ++i) {
		bep_tool::uint32 v = bh.lookup(*i);
		if (hashValue[v]) {
			fprintf(stderr, "bad hash[%s]\n", i->c_str());
		} else {
			std::string key;
			bh.lookupKey(v, key);
			if (key == *i) {
				hashValue[v] = true;
			} else {
				fprintf(stderr, "bad string[%s:%s]\n", i->c_str(), key.c_str());
			}
		}
	}
	puts("ok");
	double start = gettimeofday_sec();
	int sum = 0;
	for (StringSet::const_iterator i = all.begin(), e = all.end(); i != e; ++i) {
		sum += bh.lookup(*i);
	}
	printf("time=%f\n", gettimeofday_sec() - start);
	printf("sum=%d\n", sum);
	return 0;
}

